require 'test_helper'

class EntityTest < Test

  def pre; super
    @entity_class_name = :TestEntity
    klass = Class.new do
      extend IntegratedData::Entity
    end
    Object.send :const_set, @entity_class_name, klass
    @entity_class = Object.send :const_get, @entity_class_name
    @entity_settings = [:attribute, source: :source, id: :identifier]
  end

  def post; super
    Object.send :remove_const, @entity_class_name
    @entity_class_name = nil
    @entity_class = nil
    @entity_settings = nil
  end

  def test_fetch_defines_new_lookups
    around do
      assert_change ->{ @entity_class.lookups.length } do
        @entity_class.lookup *@entity_settings
      end
    end
  end

  def test_lookups_without_strategies_dont_repeat
    around do
      @entity_class.lookup *@entity_settings
      refute_change ->{ @entity_class.lookups.length } do
        @entity_class.lookup *@entity_settings
      end
    end
  end

  def test_lookups_with_strategies_dont_repeat
    around do
      @entity_settings.last[:strategy] = :strategy
      @entity_class.lookup *@entity_settings
      refute_change ->{ @entity_class.lookups.length } do
        @entity_class.lookup *@entity_settings
      end
    end
  end

  def test_lookups_mixing_strategies_do_repeat
    around do
      @entity_class.lookup *@entity_settings
      assert_change ->{ @entity_class.lookups.length } do
        @entity_settings.last[:strategy] = :strategy
        @entity_class.lookup *@entity_settings
      end
    end
  end

end
