require 'test_helper'

class Rxcui < String

  extend IntegratedData::Identifier

  class << self
    def parse(input)
      new input.to_s
    end
  end

  def == other
    super self.class.parse other
  end

end

class RxNorm

  extend IntegratedData::Source

  class << self
    alias_method :build, :new
  end

  def initialize(data: )
    @data = data
  end

  def method_missing(method, id, value)
    @data[method == :call ? :default : method].find do |datum|
      value == datum[id]
    end
  end

end

require 'ostruct'
class Drug < OpenStruct

  extend IntegratedData::Entity

  class << self
    def data
      {
        default: [
          {
            rxcui: 1,
            name: 'name1',
            extra_name: 'extraname1'
          },
          {
            rxcui: 2,
            name: 'name2',
            extra_name: 'extraname2'
          },
        ],
        custom: [
          {
            rxcui: 1,
            other_name: 'othername1',
            extra_name: 'extraname1'
          },
          {
            rxcui: 2,
            other_name: 'othername2',
            extra_name: 'extraname2'
          },
        ],
      }
    end
  end

  lookup :name,       source: RxNorm, id: :rxcui, identifier: Rxcui, data: data

  lookup :other_name, source: RxNorm, id: :rxcui, identifier: Rxcui, strategy: :custom, data: data

  lookup :extra_name, source: RxNorm, id: :rxcui, identifier: Rxcui, data: data do |data, attribute|
    data[attribute]
  end

end

class IntegratedDataTest < Test

  def test_default_strategy
    assert_equal 'name1', Drug.new(rxcui: 1).name
  end

  def test_custom_strategy
    assert_equal 'othername1',  Drug.new(rxcui: 1).other_name
  end

  def test_custom_extractor
    assert_equal 'extraname1',  Drug.new(rxcui: 1).extra_name
  end

end
